# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.utils.test_extract
~~~~~~~~~~~~~~~~~~~~~~~~

For testing the functions under mapillary/utils/extract.py

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Package imports
import pytest
import logging  # Logger

# Local imports
from mapillary.utils.extract import extract_properties  # Function to test
from tests.conftest import data  # Data as fixture

logger = logging.getLogger(__name__)


@pytest.mark.parametrize(
    "operation, expected", [("extract_properties(data['data'], ['id'])", 0)]
)
def test_extract_properties(data: dict, operation, expected):

    # Operation to test
    test_that = f"{operation} != {expected}"

    # Logging the intended operation to be tested
    logger.info(f"\n[test_extract_properties] Test that {test_that}")

    # Actual data on left, operation on right
    actual = extract_properties(data["data"], ["id"])

    # What was expected left, what was actual on the right
    assert len(actual["id"]) != 0, f"{test_that} failed, got {actual}"



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.__init__

This module loads the test modules under tests/

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Configuration
from . import conftest  # noqa: F401

# Utils testing
from . import utils as tests_utils  # noqa: F401

# Helper testing
from . import helper as tests_helper  # noqa: F401



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.conftest.py

This module loads variables needed to run the test

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Package imports
import pytest
import requests
import mercantile
from vt2geojson.tools import vt_bytes_to_geojson

@pytest.fixture(autouse=True)
def mly_access_token():
    """Specify the access token here"""

    # The default token is the "Testing" token specially created for testing services
    # Ref:: 'PyTest Workflow for Mapillary API v4 Python SDK'
    token = "MLY|3976749565769581|3022bf582da20b0ce0bb44373289ebb2"

    if "MLY|YYY" in token or "MLY|XXX" in token:
        raise ValueError("[tests.conftest]: MLY Access Token is not specified")

    return token


@pytest.fixture(autouse=True)
def data(mly_access_token):

    zoom, longitude, latitude = 14, 31, 30

    tile = mercantile.tile(lng=longitude, lat=latitude, zoom=zoom)

    data = vt_bytes_to_geojson(
        b_content=requests.get(
            f"https://tiles.mapillary.com/maps/vtp/mly1_public/2/14/{tile[0]}/{tile[1]}/"
            f"?access_token={mly_access_token}"
        ).content,
        x=tile.x,
        y=tile.y,
        z=tile.z,
        layer="image",
    )

    return {"data": data, "tile": tile, "longitude": longitude, "latitude": latitude}



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.utils.__init__

This module loads the modules under src/mapillary/utils for tests

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Exraction testing
from . import test_extract  # noqa: F401

# Filter testing
from . import test_filter  # noqa: F401



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
mapillary.config.api.entities
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module contains the class implementation of the Entities API endpoints
as string, for the entity API endpoint aspect of the API v4 of Mapillary.

For more information, please check out https://www.mapillary.com/developer/api-documentation/.

- Copyright: (c) 2021 Facebook
- License: MIT LICENSE
"""

# Local imports

# # Exception Handling
from mapillary.models.exceptions import InvalidFieldError


class Entities:
    """
    Each API call requires specifying the fields of the Entity you're interested in explicitly.
    A sample image by ID request which returns the id and a computed geometry could look as
    below. For each entity available fields are listed in the relevant sections. All IDs are
    unique and the underlying metadata for each entity is accessible at
    https://graph.mapillary.com/:id?fields=A,B,C. The responses are uniform and always return
    a single object, unless otherwise stated (collection endpoints). All collection endpoint
    metadata are wrapped in a {"data": [ {...}, ...]} JSON object.

    Usage::

        $ GET 'https://graph.mapillary.com/$IMAGE_ID?access_token=TOKEN&fields=id,computed_geometry'
        ... {
        ...     "id": "$IMAGE_ID",
        ...     "computed_geometry": {
        ...         "type": "Point",
        ...         "coordinates": [0, 0]
        ...     }
        ... }
    """

    @staticmethod
    def get_image(image_id: str, fields: list) -> str:
        """
        Represents the metadata of the image on the Mapillary platform with
        the following properties.

        Usage::

            >>> 'https://graph.mapillary.com/:image_id' # endpoint

        Fields::

            1. altitude - float, original altitude from Exif
            2. atomic_scale - float, scale of the SfM reconstruction around the image
            3. camera_parameters - array of float, intrinsic camera parameters
            4. camera_type - enum, type of camera projection (perspective, fisheye, or spherical)
            5. captured_at - timestamp, capture time
            6. compass_angle - float, original compass angle of the image
            7. computed_altitude - float, altitude after running image processing
            8. computed_compass_angle - float, compass angle after running image processing
            9. computed_geometry - GeoJSON Point, location after running image processing
            10. computed_rotation - enum, corrected orientation of the image
            11. exif_orientation - enum, orientation of the camera as given by the exif tag
                (see: https://sylvana.net/jpegcrop/exif_orientation.html)
            12. geometry - GeoJSON Point geometry
            13. height - int, height of the original image uploaded
            14. thumb_256_url - string, URL to the 256px wide thumbnail
            15. thumb_1024_url - string, URL to the 1024px wide thumbnail
            16. thumb_2048_url - string, URL to the 2048px wide thumbnail
            17. merge_cc - int, id of the connected component of images that were aligned together
            18. mesh - { id: string, url: string } - URL to the mesh
            19. quality_score - float, how good the image is (experimental)
            20. sequence - string, ID of the sequence
            21. sfm_cluster - { id: string, url: string } - URL to the point cloud
            22. width - int, width of the original image uploaded
        """

        fields = Entities.__field_validity(
            given_fields=fields,
            actual_fields=Entities.get_image_fields(),
            endpoint="https://graph.mapillary.com/:image_id?fields=",
        )

        return f"https://graph.mapillary.com/{image_id}/?fields={','.join(fields)}"

    @staticmethod
    def get_image_fields() -> list:
        """
        Gets list of possible image fields

        :return: Image field list
        :rtype: list
        """

        return [
            "altitude",
            "atomic_scale",
            "camera_parameters",
            "camera_type",
            "captured_at",
            "compass_angle",
            "computed_altitude",
            "computed_compass_angle",
            "computed_geometry",
            "computed_rotation",
            "exif_orientation",
            "geometry",
            "height",
            "thumb_256_url",
            "thumb_1024_url",
            "thumb_2048_url",
            "merge_cc",
            "mesh",
            "quality_score",
            "sequence",
            "sfm_cluster",
            "width",
        ]

    @staticmethod
    def get_map_feature(map_feature_id: str, fields: list) -> str:
        """
        These are objects with a location which have been derived from
        multiple detections in multiple images.

        Usage::

            >>> 'https://graph.mapillary.com/:map_feature_id' # endpoint

        Fields::

            1. first_seen_at - timestamp, timestamp of the least recent detection
                contributing to this feature
            2. last_seen_at - timestamp, timestamp of the most recent detection
                contributing to this feature
            3. object_value - string, what kind of map feature it is
            4. object_type - string, either a traffic_sign or point
            5. geometry - GeoJSON Point geometry
            6. images - list of IDs, which images this map feature was derived from
        """

        fields = Entities.__field_validity(
            given_fields=fields,
            actual_fields=Entities.get_map_feature_fields(),
            endpoint="https://graph.mapillary.com/:map_feature_id?fields=",
        )

        return (
            f"https://graph.mapillary.com/{map_feature_id}/?fields={','.join(fields)}"
        )

    @staticmethod
    def get_map_feature_fields() -> list:
        """
        Gets map feature fields

        :return: Possible map feature fields
        :rtype: list
        """

        return [
            "first_seen_at",
            "last_seen_at",
            "object_value",
            "object_type",
            "geometry",
            "images",
        ]

    @staticmethod
    def get_detection_with_image_id(
        image_id: str,
        fields: list,
    ) -> str:
        """
        Represent an object detected in a single image. For convenience
        this version of the API serves detections as collections. They can be
        requested as a collection on the resource (e.g. image) they contribute
        or belong to.

        Usage::

            >>> 'https://graph.mapillary.com/:image_id/detections'
            >>> # detections in the image with ID image_id

        Fields::

            1. created_at - timestamp, when was this detection created
            2. geometry - string, base64 encoded polygon
            3. image - object, image the detection belongs to
            4. value - string, what kind of object the detection represents
        """

        fields = Entities.__field_validity(
            given_fields=fields,
            actual_fields=Entities.get_detection_with_image_id_fields(),
            endpoint="https://graph.mapillary.com/:image_id/detections/?fields=",
        )

        return f"https://graph.mapillary.com/{image_id}/detections/?fields={','.join(fields)}"

    @staticmethod
    def get_detection_with_image_id_fields() -> list:
        """
        Gets list of possible detections for image ids

        :return: Possible detection parameters
        :rtype: list
        """

        return ["created_at", "geometry", "image", "value"]

    @staticmethod
    def get_detection_with_map_feature_id(
        map_feature_id: str,
        fields: list,
    ) -> str:
        """
        Represent an object detected in a single image. For convenience
        this version of the API serves detections as collections. They can be
        requested as a collection on the resource (e.g. map feature) they
        contribute or belong to.

        Usage::

            >>> 'https://graph.mapillary.com/:map_feature_id/detections'
            >>> # detections in the image with ID map_feature_id

        Fields::

            1. created_at - timestamp, when was this detection created
            2. geometry - string, base64 encoded polygon
            3. image - object, image the detection belongs to
            4. value - string, what kind of object the detection represents
        """

        fields = Entities.__field_validity(
            given_fields=fields,
            actual_fields=Entities.get_detection_with_map_feature_id_fields(),
            endpoint="https://graph.mapillary.com/:map_feature_id/detections/?fields=",
        )

        return f"https://graph.mapillary.com/{map_feature_id}/detections/?fields={','.join(fields)}"

    @staticmethod
    def get_detection_with_map_feature_id_fields() -> list:
        """
        Gets list of possible field parameters for map features

        :return: Map feature detection fields
        :rtype: list
        """

        return ["created_at", "geometry", "image", "value"]

    @staticmethod
    def get_organization_id(
        organization_id: str,
        fields: list,
    ) -> str:
        """
        Represents an organization which can own the imagery if users
        upload to it

        Usage::

            >>> 'https://graph.mapillary.com/:organization_id' # endpoint

        Fields::

            1. slug - short name, used in URLs
            2. name - nice name
            3. description - public description of the organization
        """

        fields = Entities.__field_validity(
            given_fields=fields,
            actual_fields=Entities.get_organization_id_fields(),
            endpoint="https://graph.mapillary.com/:organization_id?fields=",
        )

        return (
            f"https://graph.mapillary.com/{organization_id}/?fields={','.join(fields)}"
        )

    @staticmethod
    def get_organization_id_fields() -> list:
        """
        Gets list of possible organization id fields

        :return: Possible organization fields
        :rtype: list
        """

        return ["slug", "name", "description"]

    @staticmethod
    def get_sequence(
        sequence_id: str,
    ) -> str:
        """
        Represents a sequence of Image IDs ordered by capture time

        Usage::

            >>> 'https://graph.mapillary.com/image_ids?sequence_id=XXX'
            >>> # endpoint

        Fields::

            1. id - ID of the image belonging to the sequence
        """

        return f"https://graph.mapillary.com/image_ids?sequence_id={sequence_id}"

    @staticmethod
    def __field_validity(
        given_fields: list, actual_fields: list, endpoint: str
    ) -> list:
        """
        Checks if the given_fields are the actual correct fields for the given endpoint
        Compares against the list provided in `actual_fields`

        :param given_fields: The fields given as argument to check in
        :type given_fields: list

        :param actual_fields: The fields to check against
        :type actual_fields: list

        :param endpoint: The endpoint that is being targeted
        :type endpoint: str

        :raises InvalidFieldError: Raised when an API endpoint is passed invalid field elements

        :return: The given_fields if everything is correct
        :rtype: list
        """

        # Converting the given_fields into lowercase
        given_fields = [field.lower() for field in given_fields]

        # Enforce the existence of 'geometry' in the given_fields
        if "geometry" not in given_fields:
            given_fields.append("geometry")

        # Going through all the given fields
        for field in given_fields:

            # If 'all' is encountered ...
            if field == "all":

                # ... simply return the actual_fields list
                return actual_fields

            # If a field does not exist in the actual_fields ...
            if field not in actual_fields:

                # Raise an InvalidFieldError error
                raise InvalidFieldError(
                    # Specifying what endpoint was specified
                    endpoint=endpoint,
                    # Specify what field triggered the exception
                    field=field,
                )

        # If no error occurred, and all the fields are correct, return the given_fields
        return given_fields



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.helper.client
~~~~~~~~~~~~~~~~~~~

For pulling in data for test purposes from the Mapillary APIv4. Some higher level requirements
use a geojson or a bbox as the query extent for further testing. This file serves to be an
easy way of pulling in data from endpoints through an easy CLI instead of writing Python
code as 1 offs for getting the data and parsing it.

Contributions welcome!

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Local imports
import fetch

# Package imports
import inspect
import logging
import json
import sys
import os

colors = {
    "HEADER": "\033[95m",
    "OKBLUE": "\033[94m",
    "OKCYAN": "\033[96m",
    "OKGREEN": "\033[92m",
    "WARNING": "\033[93m",
    "FAIL": "\033[91m",
    "ENDC": "\033[0m",
    "BOLD": "\033[1m",
    "UNDERLINE": "\033[4m",
}


def list_fetched_functions(fetched_functions: list) -> None:
    """Simply lists out the functions described in fetch.py along with their DocString

    :param fetched_functions: List of function names along with their references in a tuple set
    :type fetched_functions: list

    :return: Nothing to return
    :rtype: None
    """

    # Through each tuple in the format (name, reference)
    for index, function in enumerate(fetched_functions):

        # Print out ...
        print(
            # ... an index for the function, then the name of the function in cyan color ...
            f"{index + 1}. {colors['OKCYAN']}{function[0]}{colors['ENDC']} "
            # ... then print the docstring as well
            f"- {function[1].__doc__}"
        )


def save_as_geojson(data: dict) -> None:
    """Saves the resulting geojson dictionary in a file tagged .geojson

    :param data: The data to save
    :type data: dict

    :return: Nothing to return
    :rtype: None
    """

    # TODO: Test if this works perfectly fine in Windows, since it uses the `os` package

    # Save the file at the given path, create it if it does not exist
    with open(
        f'{os.path.abspath(".")}/data/{sys.argv[2]}.geojson', mode="w"
    ) as geojson_file:

        # Save the dictionary as a json in a clean formatted way
        json.dump(data, geojson_file, sort_keys=True, indent=4)


def main():
    """Main logic for the CLI"""

    # Getting the list of functions and their references from fetch.py
    fetched_functions = inspect.getmembers(fetch, inspect.isfunction)

    # If there are insufficient arguments
    if len(sys.argv) < 3:

        # Log an error ...
        logging.error(
            # ... indicating the source of the error ...
            " - client.py\n"
            # ... the reason for the error ...
            f"{colors['FAIL']}Invalid format!{colors['ENDC']}\n"
            # ... giving the user a format for the colors ...
            f"Try: python client.py {colors['BOLD']}'MLY|XXX'{colors['ENDC']} "
            # ... format of the function called
            f"{colors['OKCYAN']}fetch_function_here{colors['ENDC']}\n"
        )

        # Finally, list the functions for the user
        list_fetched_functions(fetched_functions=fetched_functions)

    else:

        # Getting the access token
        access_token = str(sys.argv[1])

        # result output variable
        result: dict = {}

        # Going through each tuple of the format (func_name, func_reference)
        for function in fetched_functions:

            # If the function named is what is asked through the args
            if function[0] == str(sys.argv[2]):

                # Get the resulting output from the reference
                result = function[1](access_token=access_token)

        # If the results ends up empty
        if result == {}:

            # Log an error ...
            logging.error(
                # ... indicating the source of the error ...
                " - client.py\n"
                # ... the reason for the error
                f"{colors['FAIL']}Unrecognized function call!{colors['ENDC']}\n"
            )

            # Finally, list the functions for the user
            list_fetched_functions(fetched_functions=fetched_functions)

        # Saving the data
        save_as_geojson(result)


if __name__ == "__main__":
    main()



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.helper.fetch
~~~~~~~~~~~~~~~~~~

This file contains the set of functions that are called by the client.py
in response to a query.

Contributions welcome!

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Package imports
import mercantile, requests
import vt2geojson.tools


def fetch_map_feature_point(access_token: str):
    """For fetching utility poles and street lights within a bounding using the Mercantile/Mapbox
    Vector Tile library

    Reference::
        1. 'https://blog.mapillary.com/update/2021/06/23/getting-started-with-the-new-mapillary-api-v4.html'
    """

    # define an empty geojson as output
    output = {"type": "FeatureCollection", "features": []}

    # a bounding box in [east_lng,_south_lat,west_lng,north_lat] format
    east, south, west, north = [
        -80.13423442840576,
        25.77376933762778,
        -80.1264238357544,
        25.788608487732198,
    ]

    filter_values = ["object--support--utility-pole", "object--street-light"]
    # get the tiles with x and y coors intersecting bbox at zoom 14 only
    tiles = list(mercantile.tiles(east, south, west, north, 14))

    # loop through all tiles to get IDs of Mapillary data
    for tile in tiles:

        # Creating the tile URL
        tile_url = (
            f"https://tiles.mapillary.com/maps/vtp/mly_map_feature_point/2/"
            f"{tile.z}/{tile.x}/{tile.y}?access_token={access_token}"
        )

        response = requests.get(tile_url)

        data = vt2geojson.tools.vt_bytes_to_geojson(
            response.content, tile.x, tile.y, tile.z
        )

        filtered_data = [
            feature
            for feature in data["features"]
            if feature["properties"]["value"] in filter_values
        ]

        for feature in filtered_data:

            if (
                feature["geometry"]["coordinates"][0] > east
                and feature["geometry"]["coordinates"][0] < west
            ) and (
                feature["geometry"]["coordinates"][1] > south
                and feature["geometry"]["coordinates"][1] < north
            ):

                output["features"].append(feature)

    return output



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
setup.py

This file implements the configuration for uploading the projec to PyPi

Reference::
    1. https://packaging.python.org/tutorials/packaging-projects/
    2. https://www.youtube.com/watch?v=GIF3LaRqgXo

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Packages
import setuptools  # setup, find_packages, Command
import os
import sys
import json
import shutil

# Setup variables. Change as needed
NAME = "mapillary"  
VERSION = "1.0.0"
AUTHOR = "Christopher Beddow"
AUTHOR_EMAIL = "support@mapillary.zendesk.com"
LICENSE = "MIT"
PLATFORM = ["POSIX", "MacOS X", "Linux", "Windows"]
DESCRIPTION = (
    "A Python 3 library built on the Mapillary API v4 to facilitate retrieving and "
    "working with Mapillary data"
)
LONG_DESCRIPTION_CONTENT_TYPE = "text/markdown"
URL = "https://github.com/mapillary/mapillary-python-sdk"
REQUIRES_PYTHON = ">=3.0"
HERE = os.path.abspath(os.path.dirname(__file__))
REQUIREMENTS = [
    "mercantile",
    "mapbox-vector-tile",
    "pytest",
    "vt2geojson",
    "shapely",
    "turfpy",
]
CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
    "Topic :: Scientific/Engineering :: GIS",
    "Topic :: Scientific/Engineering :: Information Analysis",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Microsoft",
    "Operating System :: MacOS",
    "Operating System :: POSIX",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
]
PROJECT_URLS = {
    "Download": "https://pypi.org/project/mapillary/#files",
    "Release Notes": "https://github.com/mapillary/mapillary-python-sdk/releases",
    "Bug Tracker": "https://github.com/mapillary/mapillary-python-sdk/issues",
    "Source": "https://github.com/mapillary/mapillary-python-sdk",
    "Twitter": "https://twitter.com/mapillary",
    "Developer Resources": "https://www.mapillary.com/developer",
    "Community Forum": "https://forum.mapillary.com/",
    "Blog": "https://blog.mapillary.com/",
    "Facebook": "https://www.facebook.com/mapillary/",
    "Website": "https://www.mapillary.com/",
}
PACKAGE_DIR = {"": "src"}

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with open(os.path.join(HERE, "README.md"), encoding="utf-8") as f:
        LONG_DESCRIPTION = "\n" + f.read()
except FileNotFoundError:
    LONG_DESCRIPTION = DESCRIPTION


def locked_requirements(section):
    """Look through the 'Pipfile.lock' to fetch requirements by section."""
    with open("Pipfile.lock") as pip_file:
        pipfile_json = json.load(pip_file)

    if section not in pipfile_json:
        print("{0} section missing from Pipfile.lock".format(section))
        return []

    return [
        package + detail.get("version", "")
        for package, detail in pipfile_json[section].items()
    ]


class UploadCommand(setuptools.Command):
    """Support setup.py upload."""

    description = "Build and publish the package."
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status("Removing previous builds…")
            shutil.rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass

        self.status("Building Source and Wheel (universal) distribution…")
        os.system(f"{sys.executable} setup.py sdist bdist_wheel --universal")

        self.status("Uploading the package to PyPI via Twine…")
        os.system("twine upload dist/*")

        self.status("Pushing git tags…")
        os.system(f"git tag v{VERSION}")
        os.system("git push --tags")

        sys.exit()


# Where the magic happens
setuptools.setup(
    # METADATA
    # # Package name
    name=NAME,
    # # Package versioning
    version=VERSION,
    # # Author name(s)
    author=AUTHOR,
    # # License
    license=LICENSE,
    # # Platform specification
    platforms=PLATFORM,
    # # Author email(s)
    author_email=AUTHOR_EMAIL,
    # # Short description about the package
    description=DESCRIPTION,
    # # Long descirption about the package
    long_description=LONG_DESCRIPTION,
    # # Content type of the long description
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    # # URL to the project on GitHub
    url=URL,
    # # Dictionary of useful project related URLs
    project_urls=PROJECT_URLS,
    # Codebase, Wheel, Egg Setup
    # Setuptools for finding other needed packages from Pipfile
    packages=setuptools.find_packages(where="src"),
    # # Specifiy the package directory
    package_dir=PACKAGE_DIR,
    # # A string or list of strings specifying what other distributions need to be installed
    # # when this one is
    install_requires=locked_requirements("default"),
    # # What Python version is required
    python_requires=REQUIRES_PYTHON,
    # # What package data to include
    include_package_data=True,
    # # List of classifiers for metadata on PyPI
    classifiers=CLASSIFIERS,
    # # What commands to run
    cmdclass={
        "upload": UploadCommand,
    },
    # # A dictionary mapping names of “extras” (optional features of your project) to strings or
    # # lists of strings specifying what other distributions must be installed to support those
    # # features
    # # Right now, this fetches development dependencies which are un-needed
    # extras_require={"dev": locked_requirements("develop")},
)



# Copyright (c) Facebook, Inc. and its affiliates. (http://www.facebook.com)
# -*- coding: utf-8 -*-

"""
tests.utils.test_filter
~~~~~~~~~~~~~~~~~~~~~~~~

For testing the functions under mapillary/utils/filter.py

:copyright: (c) 2021 Facebook
:license: MIT LICENSE
"""

# Package imports
import pytest
import logging  # Logger

# Local imports
from mapillary.utils.filter import pipeline  # Pipeline
from tests.conftest import data  # Data as fixture

logger = logging.getLogger(__name__)


@pytest.mark.parametrize(
    "operation, expected",
    [("pipeline(data=fetched_data, components=[...]", "len(data['features'])")],
)
def test_pipeline_min_captured_at(data: dict, operation, expected):

    # Operation to test
    test_that = f"len({operation}) != {expected}"

    # Logging the intended operation to be tested
    logger.info(f"\n[test_pipeline_min_captured_at] Test that {test_that}")

    # Get data from data fixture
    fetched_data, _, longitude, latitude = data.values()

    # Pass through pipeline. Actual data on left, operation on right
    actual = pipeline(
        data=fetched_data,
        components=[
            {
                "filter": "haversine_dist",
                "radius": 5000,
                "coords": [longitude, latitude],
            },
            {
                "filter": "min_captured_at",
                "min_timestamp": "2020-01-01",
            },
        ],
    )

    # What was expected left, what was actual on the right
    assert len(actual) != len(
        fetched_data["features"]
    ), f"{test_that} failed, got {actual}"


@pytest.mark.parametrize(
    "operation, expected",
    [("pipeline(data=fetched_data, components=[...]", "len(data['features'])")],
)
def test_pipeline_haversine_dist(data: dict, operation, expected):

    # Operation to test
    test_that = f"len({operation}) != {expected}"

    # Logging the intended operation to be tested
    logger.info(f"\n[test_pipeline_haversine_dist] Test that {test_that}")

    # Get data from data fixture
    fetched_data, _, longitude, latitude = data.values()

    # Pass through pipeline. Actual data on left, operation on right
    actual = pipeline(
        data=fetched_data,
        components=[
            {
                "filter": "haversine_dist",
                "radius": 5000,
                "coords": [longitude, latitude],
            },
            {
                "filter": "haversine_dist",
                "radius": 1000,
                "coords": [longitude, latitude],
            },
        ],
    )

    # What was expected left, what was actual on the right
    assert len(actual) != len(
        fetched_data["features"]
    ), f"{test_that} failed, got {actual}"
